// Demonstrate a recursion-free version of printing all numbers that
// sum to 8. Requires use of an explicit stack. Create an auxiliary
// class SumFrame to represent the state of the computation.
import java.util.Stack;

class SumFrame{
  public int current;           // Current sum
  public Stack<Integer> toAdd;  // Numbers remaining to add
  public String history;        // History of adds that led here

  // Create a new frame, populate toAdd with numbers that sum to <=
  // the target value
  public SumFrame(int current, int target, String history){
    this.current = current;
    this.history = history;
    this.toAdd = new Stack<Integer>();
    int remaining = target - current;
    for(int i=remaining; i>=1; i--){
      this.toAdd.push(i);
    }
  }
  public String toString(){
    return String.format("cur: %d hist: '%s' toAdd: %s",
                         current, history,toAdd.toString());
  }
}


public class SumsNoRecursion{

  // Simulate the recursion by setting up a stack of "calls" to generate sums
  public static void totalsTarget(int target){
    Stack<SumFrame> stack = new Stack<SumFrame>();
    SumFrame first = new SumFrame(0,target,"");
    stack.push(first);

    // Simulate the recursive call stack with a loop
    while(stack.size() > 0){
      SumFrame frame = stack.peek();
      System.out.println(frame);
      if(frame.current == target){ // "Base" case - print history
        System.out.printf("%d = %s\n",target,frame.history);
        stack.pop();
      }
      else if(frame.toAdd.size() == 0){ // Emptied this frame out nothing left to do with it
        stack.pop();
      }
      else{                     // More work to do with this frame
        int nextAdd = frame.toAdd.pop();
        int nextCurrent = frame.current + nextAdd;
        String nextHistory = frame.history + " " + nextAdd;
        SumFrame newFrame = new SumFrame(nextCurrent, target, nextHistory);
        stack.push(newFrame);
      }
    }
  }
        
  public static void main(String args[]){
    totalsTarget(8);
  }
}
